# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['bad_cases', 'Graph_free_market', 'Graph_monopoly']

# %% ../nbs/00_core.ipynb 3
import matplotlib.pyplot as plt
from sympy import *
import sympy
import numpy as np
import random
import math

class Graph_free_market:
    def __init__(self) -> None:
        pass
        
    
    def __str__(self):
        return str(self.__class__) + ": " + str(self.__dict__)
    
    def market_graph(self, supply: str, demand: str, complete = False) -> None:
        price = self.get_price(supply, demand)
        quantity = self.get_quantity(supply, demand)
        
        start = 0
        if "x" in demand:
            end = math.floor(self.get_zero_point(demand))
            print(f"end is {end}")
        else:
            end = 1.5 * math.floor(quantity)
        step = 1 
        

        if "x" in supply:
            supply_dict = self.get_calculate_values(supply, end)
            print(f"supply_dict{supply_dict}")
            supply_curve = list(supply_dict.values())


            plt.plot(supply_dict.keys(), supply_dict.values(), label = "Supply") 
            
        else:
            supply_list = [float(price) for i in range(start, math.ceil(end), step)]
            supply_curve = supply_list
            plt.plot(supply_curve, label = "Supply") 
            
        
        if "x" in demand:
            demand_dict = self.get_calculate_values(demand, end)
            print(f"demand_dict{supply_dict}")
            demand_curve = list(demand_dict.values())
            plt.plot(demand_dict.keys(),demand_dict.values(), label = "Demand") 
            
        else:
            demand_list = [float(price) for i in range(start, math.ceil(end), step)]
            demand_curve = demand_list
            plt.plot(demand_curve, label = "Demand") 
            

        plt.xlabel("Quantity")
        plt.ylabel("Price")
        
        if complete == True:
            
            x_range = [i for i in range(0, math.floor(quantity))] + [float(quantity)]
            if len(x_range) <= 1:
                x_range = [i for i in range(0, math.ceil(quantity))] + [float(quantity)]

            y_range = [i for i in range(0, math.floor(price))] + [float(price)]
            if len(y_range) <= 1:
                y_range = [i for i in range(0, math.ceil(price))] +[float(price)]


            price_curve = np.array([price for i in range(len(x_range))], dtype=float) 
            quantum_curve = np.array([quantity for i in range(len(y_range))], dtype=float) 

            plt.plot(x_range,                              # x [i for i in range(len(price_curve))],
                     price_curve,                          # y
                     linestyle = "dashed", label = f"Price*: {price}")
            
            plt.plot(quantum_curve,                             # x 
                     y_range,                                   # y [i for i in range(len(quantum_curve))]
                     linestyle = "dashed", label = f"Quantity*: {quantity}")
            
            

            if "x" in supply:
                price_curve = np.array(price_curve)
                supply_curve = np.array(supply_curve[0:len(price_curve)])  
                
                x_range = np.array(x_range)
                
                # Create a valid boolean array for the 'where' condition
                condition = supply_curve[0:len(price_curve)] <= price_curve

                plt.fill_between(x_range, supply_curve, price_curve, where = condition, color = "silver", alpha=0.9) # producer surplus

                plt.text(float(quantity/4), price - 1, "Producer\nsurplus")
                

            if "x" in demand:
                price_curve = np.array(price_curve)
                demand_curve = np.array(demand_curve[0:len(price_curve)])  
                
                x_range = np.array(x_range)

                condition = demand_curve[0:len(price_curve)] >= price_curve
            
                plt.fill_between(x_range, demand_curve, price_curve, where = condition, color = "lightgrey", alpha=0.9) # consumer surplus

                plt.text(float(quantity/4), price + 1, "Consumer\nsurplus")
            
            print(f"x_range is: {x_range}\nsupply curve is:{supply_curve}\ndemand curve is: {demand_curve}\nprice curve is: {price_curve}")

        plt.legend() 
        plt.show()


    def get_calculate_values(self, expression: str, end: int) -> dict:
        start = 0
        step = 1 

        quantity = end
        end = math.ceil(end)

        value_pairs = {}
        equation_function = self.create_equation_function(expression)

        
        if equation_function:
            x_values = sorted([i for i in range(start, end, step)] + [quantity])
            for x_val in x_values:
                result = float(equation_function(x_val))
                value_pairs[x_val] = result
            #result = float(equation_function(quantity))
            print(f"result from rational is {result}")
            #value_pairs[x_val] = result
            if end <= 1:
                x_values = sorted([i for i in range(start, end, step)] + [quantity])
                for x_val in x_values:
                    result = float(equation_function(x_val))
                    value_pairs[x_val] = result
                result = float(equation_function(quantity))
                print(f"result from rational is {result}")
                value_pairs[x_val] = result


            else:
                x_values = sorted([i for i in range(start, end, step)] + [quantity])
                for x_val in x_values:
                    result = float(equation_function(x_val))
                    value_pairs[x_val] = result
                
                
               # print(f"For x = {x_val}, the result is {result}")

        else:
            print("Error: Unable to create the equation function.")
        return value_pairs    
    

    def create_equation_function(self, equation_str: str) -> str:
        x = symbols('x')
        
        try:
            equation = parse_expr(equation_str)
            equation_function = lambda x_val: equation.subs(x, x_val)
            return equation_function
        except Exception as e:
            return None


    def get_zero_point(self, expression: str) -> float:
        x = symbols('x')
        
        # Create the equation from the supply and demand functions
        equation = parse_expr(expression)
        
        # Calculate the equilibrium price and quantity
        zero_point = solve(equation, x)[0]
        print(f"zero_points {zero_point}")
        #print(zero_point)
        return zero_point
    
    
    def get_quantity(self, supply: str, demand: str) -> float:
        x = symbols('x')
        
        # Create the equation from the supply and demand functions
        supply_eq = parse_expr(supply)
        demand_eq = parse_expr(demand)
        
        # Calculate the equilibrium price and quantity
        quantity = max(solve(Eq(supply_eq, demand_eq), x))
        #print(quantity)
        return quantity
    

    def get_price(self, supply: str, demand: str) -> float:
        x, y = symbols('x y')
        quantity = self.get_quantity(supply, demand)
        
        if "x" in demand:
            end = self.get_zero_point(demand)
        else:
            end = 2 * quantity
            
        # Create the equation from the supply and demand functions
        """ supply_eq = parse_expr(supply)
        demand_eq = parse_expr(demand) """
        
        equation_function = self.create_equation_function(demand)
        
        
        if "x" not in supply:
            price = float(supply)
        elif "x" not in demand:
            price = float(demand)
        else:
            price = equation_function(quantity)
             
        print(f"price is around {round(price, 3)}")
        return price


    def get_consumer_surplus(self, supply: str, demand: str) -> float:
        x, y = symbols('x y')
        
        # Create the equation from the supply and demand functions
        """ supply_eq = parse_expr(supply)
        demand_eq = parse_expr(demand) """
        
        # Calculate the equilibrium price and quantity
        price = self.get_price(supply, demand)
        quantity = self.get_quantity(supply, demand)
        
        # Define the inverse demand function (price as a function of quantity)
        consumer_surplus = parse_expr(f"{demand}-{price}")
        
        # Calculate consumer surplus
        surplus = sympy.integrate(consumer_surplus, (x, 0, quantity)) 
        
        return surplus
    

    def get_producer_surplus(self, supply: str, demand: str) -> float:
        x, y = symbols('x y')
        
        # Create the equation from the supply and demand functions
        supply_eq = parse_expr(supply)
        demand_eq = parse_expr(demand)
        
        # Calculate the equilibrium price and quantity
        price = self.get_price(supply, demand)
        quantity = self.get_quantity(supply, demand)
        
        # Define the inverse demand function (price as a function of quantity)
        """ if "x" in supply:
            inverse_supply = solve(supply_eq - y, x)[0]
        else:
            inverse_supply = parse_expr(f"{supply}")
            
        if "x" in demand:    
            inverse_demand = solve(demand_eq - y, x)[0]
        else:
            inverse_demand = parse_expr(f"{demand}") """
        
        
        producer_surplus = parse_expr(f"{price}-{supply}")
            
        
        # Calculate consumer surplus
        surplus = sympy.integrate(producer_surplus, (y, 0, quantity)) 
        
        return surplus


    def get_economic_surplus(self, supply: str, demand: str) -> float:
        consumer = self.get_consumer_surplus(supply, demand)
        producer = self.get_producer_surplus(supply, demand)
        economic_surplus = consumer + producer
        return economic_surplus
    
    
    
if __name__ == "__main__":
    
        for i in range(0, 2):
            a = random.randint(1,5)
            b = random.randint(5,10)
            c = random.randint(1,5)
            d = random.randint(0,1)
            
            graph = Graph_free_market()
            supply_function = f"{d} + {c} * x"
            demand_function = f"{b} - {a} * x"

            print(f"supply function {supply_function}\ndemand function {demand_function}")
            graph.market_graph(supply_function, demand_function, complete=True)

        """ graph = Graph_free_market()
        supply_function = f"{0} + {4} * x"
        demand_function = f"{5} - {1} * x"
        graph.market_graph(supply_function, demand_function, complete=True) """
        
        """ print(f"supply function {supply_function}\ndemand function {demand_function}")
        
        consumer_surplus = graph.get_consumer_surplus(supply_function, demand_function)
        print("Consumer Surplus:", consumer_surplus)

        producer_surplus = graph.get_producer_surplus(supply_function, demand_function)
        print("Producer Surplus:", producer_surplus)

        economic_surplus = graph.get_economic_surplus(supply_function, demand_function)

        print("Economic Surplus:", economic_surplus)
        
        graph.market_graph(supply_function, demand_function, complete=True) """
        
        """ price = graph.get_price(supply_function, demand_function)
        quantity = graph.get_quantity(supply_function, demand_function)
        print(f"Price: {price}, Quantity: {quantity}")
         """
        """ graph = Graph_monopoly()
        supply = "2"
        demand = "50 -  x**2"
        graph.market_graph(supply, demand,0, 12, 1, complete=True, is_tot_cost = False) """


bad_cases = ["supply function 1 + 5 * x demand function 5 - 3 * x",
             "supply function 0 + 5 * x demand function 10 - 1 * x",
             "supply function 0 + 4 * x demand function 5 - 0 * x",
             "supply function 0 + 1 * x demand function 9 - 5 * x"]

# %% ../nbs/00_core.ipynb 4
import matplotlib.pyplot as plt
from sympy import *
import sympy
class Graph_monopoly(Graph_free_market):
    def __init__(self) -> None:
        super().__init__()
        
    
    def market_graph(self, supply: str, demand: str, complete = False, is_tot_cost = False) -> None:
        x = symbols('x')
        
        if is_tot_cost == True:
            supply_parsed = parse_expr(supply)
            marginal_cost = diff(supply_parsed, x)
        
        demand_parsed = parse_expr(demand)
        marginal_revenue = diff(demand_parsed, x)
        
        
        
        supply, demand =str(marginal_cost), str(marginal_revenue)
        
        
        
        price = self.get_price(supply, demand)
        quantity = self.get_quantity(supply, demand)

        supply = self.get_calculate_values(supply, start, end, step)
        demand = self.get_calculate_values(demand, start, end, step)

        if complete == True:
            plt.plot([i for i in range(0, round(quantity) + 1)], [price for i in range(0, round(quantity) + 1)],
                     linestyle = "dashed", label = f"Price*: {price}")
            plt.plot([quantity for i in range(0, round(price) + 1 )], [i for i in range(0,round(price) + 1)],
                     linestyle = "dashed", label = f"Quantity*: {quantity}")


     
        plt.plot(supply.keys(),supply.values(), label = "Supply") 
        plt.plot(demand.keys(),demand.values(), label = "Demand") 

        

        plt.xlabel("Quantity")
        plt.ylabel("Price")

        plt.legend() 
        plt.show()
    
        
        # return super().market_graph(supply, demand, start, end, step, complete)
        


